-- It is based on a telnet server  T. Ellison,  June 2019
-- rewritten for esp32 by serg3295, May 2023

local modname = ...
local net, node, io, json = net, node, io, sjson

---@class telnet_cfg
---@field port number
---@field timeout integer
local cfg = {}

-- Read settings from JSON file
---@param fn string
---@return telnet_cfg
local function getSettings(fn)
  assert(io.input(fn), "cannot open file 'telnet.json'")
  cfg = json.decode(io.read("a"))
  io.close(io.input())

  return cfg and cfg or error("cannot get telnet config")
end

---@param skt socket
local function telnetSession(skt)
  local con_std = skt

  local function onDisconnect()
    node.output()
    ---@diagnostic disable-next-line: cast-local-type
    skt = nil
  end

  ---Sends data to remote peer
  ---@param str string
  local function output_CB(str) -- upval: socket
    if str and #str > 0 and con_std ~= nil then
      con_std:send(str)
    end
  end

  local firstCall = true
  local function onReceive(_, str)
    if firstCall then
      firstCall = false
      str = '\r\n'
    end
    node.input(str)
  end

  node.output(output_CB, 0)
  skt:on("disconnection", onDisconnect)
  skt:on("receive",onReceive)
  print(("Welcome to NodeMCU world (%d mem free)"):format(node.heap()))
end

---start up telnet server
---@return table server a net.tcp.server
local function startup()
  local srv = net.createServer(net.TCP, cfg.timeout)
  srv:listen(cfg.port, telnetSession)
  print(("listening on port %d"):format(cfg.port))
  return srv
end

---creates telnet server
return function()
  package.loaded[modname] = nil

  cfg = getSettings('telnet.json')
  local telnet = startup()
  assert(telnet, "cannot create telnet server")
end
